@@ -2,37 +2,42 @@
 Backends for embarrassingly parallel code.
 """
 
+import contextlib
 import gc
 import os
-import warnings
 import threading
-import contextlib
+import warnings
 from abc import ABCMeta, abstractmethod
 
+from ._multiprocessing_helpers import mp
 from ._utils import (
+    _retrieve_traceback_capturing_wrapped_call,
     _TracebackCapturingWrapper,
-    _retrieve_traceback_capturing_wrapped_call
 )
 
-from ._multiprocessing_helpers import mp
-
 if mp is not None:
-    from .pool import MemmappingPool
     from multiprocessing.pool import ThreadPool
+
     from .executor import get_memmapping_executor
 
     # Import loky only if multiprocessing is present
-    from .externals.loky import process_executor, cpu_count
+    from .externals.loky import cpu_count, process_executor
     from .externals.loky.process_executor import ShutdownExecutorError
+    from .pool import MemmappingPool
 
 
 class ParallelBackendBase(metaclass=ABCMeta):
     """Helper abc which defines all methods a ParallelBackend must implement"""
 
-    supports_inner_max_num_threads = False
-    supports_retrieve_callback = False
     default_n_jobs = 1
 
+    supports_inner_max_num_threads = False
+
+    # This flag was introduced for backward compatibility reasons.
+    # New backends should always set it to True and implement the
+    # `retrieve_result_callback` method.
+    supports_retrieve_callback = False
+
     @property
     def supports_return_generator(self):
         return self.supports_retrieve_callback
@@ -43,16 +48,22 @@ class ParallelBackendBase(metaclass=ABCMeta):
 
     nesting_level = None
 
-    def __init__(self, nesting_level=None, inner_max_num_threads=None,
-                 **kwargs):
-        super().__init__(**kwargs)
+    def __init__(
+        self, nesting_level=None, inner_max_num_threads=None, **backend_kwargs
+    ):
+        super().__init__()
         self.nesting_level = nesting_level
         self.inner_max_num_threads = inner_max_num_threads
+        self.backend_kwargs = backend_kwargs
 
     MAX_NUM_THREADS_VARS = [
-        'OMP_NUM_THREADS', 'OPENBLAS_NUM_THREADS', 'MKL_NUM_THREADS',
-        'BLIS_NUM_THREADS', 'VECLIB_MAXIMUM_THREADS', 'NUMBA_NUM_THREADS',
-        'NUMEXPR_NUM_THREADS',
+        "OMP_NUM_THREADS",
+        "OPENBLAS_NUM_THREADS",
+        "MKL_NUM_THREADS",
+        "BLIS_NUM_THREADS",
+        "VECLIB_MAXIMUM_THREADS",
+        "NUMBA_NUM_THREADS",
+        "NUMEXPR_NUM_THREADS",
     ]
 
     TBB_ENABLE_IPC_VAR = "ENABLE_IPC"
@@ -75,20 +86,77 @@ class ParallelBackendBase(metaclass=ABCMeta):
         as long as all the workers have enough work to do.
         """
 
-    @abstractmethod
     def apply_async(self, func, callback=None):
-        """Schedule a func to be run"""
+        """Deprecated: implement `submit` instead."""
+        raise NotImplementedError("Implement `submit` instead.")
+
+    def submit(self, func, callback=None):
+        """Schedule a function to be run and return a future-like object.
+
+        This method should return a future-like object that allow tracking
+        the progress of the task.
+
+        If ``supports_retrieve_callback`` is False, the return value of this
+        method is passed to ``retrieve_result`` instead of calling
+        ``retrieve_result_callback``.
+
+        Parameters
+        ----------
+        func: callable
+            The function to be run in parallel.
+
+        callback: callable
+            A callable that will be called when the task is completed. This callable
+            is a wrapper around ``retrieve_result_callback``. This should be added
+            to the future-like object returned by this method, so that the callback
+            is called when the task is completed.
+
+            For future-like backends, this can be achieved with something like
+            ``future.add_done_callback(callback)``.
+
+        Returns
+        -------
+        future: future-like
+            A future-like object to track the execution of the submitted function.
+        """
+        warnings.warn(
+            "`apply_async` is deprecated, implement and use `submit` instead.",
+            DeprecationWarning,
+        )
+        return self.apply_async(func, callback)
 
     def retrieve_result_callback(self, out):
-        """Called within the callback function passed in apply_async.
+        """Called within the callback function passed to `submit`.
 
-        The argument of this function is the argument given to a callback in
-        the considered backend. It is supposed to return the outcome of a task
-        if it succeeded or raise the exception if it failed.
+        This method can customise how the result of the function is retrieved
+        from the future-like object.
+
+        Parameters
+        ----------
+        future: future-like
+            The future-like object returned by the `submit` method.
+
+        Returns
+        -------
+        result: object
+            The result of the function executed in parallel.
         """
 
-    def configure(self, n_jobs=1, parallel=None, prefer=None, require=None,
-                  **backend_args):
+    def retrieve_result(self, out, timeout=None):
+        """Hook to retrieve the result when support_retrieve_callback=False.
+
+        The argument `out` is the result of the `submit` call. This method
+        should return the result of the computation or raise an exception if
+        the computation failed.
+        """
+        if self.supports_timeout:
+            return out.get(timeout=timeout)
+        else:
+            return out.get()
+
+    def configure(
+        self, n_jobs=1, parallel=None, prefer=None, require=None, **backend_kwargs
+    ):
         """Reconfigure the backend and return the number of workers.
 
         This makes it possible to reuse an existing backend instance for
@@ -113,10 +181,6 @@ class ParallelBackendBase(metaclass=ABCMeta):
     def batch_completed(self, batch_size, duration):
         """Callback indicate how long it took to run a batch"""
 
-    def get_exceptions(self):
-        """List of exception types to be captured."""
-        return []
-
     def abort_everything(self, ensure_ready=True):
         """Abort any running tasks
 
@@ -148,29 +212,12 @@ class ParallelBackendBase(metaclass=ABCMeta):
         nesting. Beyond, switch to sequential backend to avoid spawning too
         many threads on the host.
         """
-        nesting_level = getattr(self, 'nesting_level', 0) + 1
+        nesting_level = getattr(self, "nesting_level", 0) + 1
         if nesting_level > 1:
             return SequentialBackend(nesting_level=nesting_level), None
         else:
             return ThreadingBackend(nesting_level=nesting_level), None
 
-    @contextlib.contextmanager
-    def retrieval_context(self):
-        """Context manager to manage an execution context.
-
-        Calls to Parallel.retrieve will be made inside this context.
-
-        By default, this does nothing. It may be useful for subclasses to
-        handle nested parallelism. In particular, it may be required to avoid
-        deadlocks if a backend manages a fixed number of workers, when those
-        workers may be asked to do nested Parallel calls. Without
-        'retrieval_context' this could lead to deadlock, as all the workers
-        managed by the backend may be "busy" waiting for the nested parallel
-        calls to finish, but the backend has no free workers to execute those
-        tasks.
-        """
-        yield
-
     def _prepare_worker_env(self, n_jobs):
         """Return environment variables limiting threadpools in external libs.
 
@@ -200,6 +247,23 @@ class ParallelBackendBase(metaclass=ABCMeta):
             env[self.TBB_ENABLE_IPC_VAR] = "1"
         return env
 
+    @contextlib.contextmanager
+    def retrieval_context(self):
+        """Context manager to manage an execution context.
+
+        Calls to Parallel.retrieve will be made inside this context.
+
+        By default, this does nothing. It may be useful for subclasses to
+        handle nested parallelism. In particular, it may be required to avoid
+        deadlocks if a backend manages a fixed number of workers, when those
+        workers may be asked to do nested Parallel calls. Without
+        'retrieval_context' this could lead to deadlock, as all the workers
+        managed by the backend may be "busy" waiting for the nested parallel
+        calls to finish, but the backend has no free workers to execute those
+        tasks.
+        """
+        yield
+
     @staticmethod
     def in_main_thread():
         return isinstance(threading.current_thread(), threading._MainThread)
@@ -220,10 +284,10 @@ class SequentialBackend(ParallelBackendBase):
     def effective_n_jobs(self, n_jobs):
         """Determine the number of jobs which are going to run in parallel"""
         if n_jobs == 0:
-            raise ValueError('n_jobs == 0 in Parallel has no meaning')
+            raise ValueError("n_jobs == 0 in Parallel has no meaning")
         return 1
 
-    def apply_async(self, func, callback=None):
+    def submit(self, func, callback=None):
         """Schedule a func to be run"""
         raise RuntimeError("Should never be called for SequentialBackend.")
 
@@ -247,7 +311,7 @@ class PoolManagerMixin(object):
     def effective_n_jobs(self, n_jobs):
         """Determine the number of jobs which are going to run in parallel"""
         if n_jobs == 0:
-            raise ValueError('n_jobs == 0 in Parallel has no meaning')
+            raise ValueError("n_jobs == 0 in Parallel has no meaning")
         elif mp is None or n_jobs is None:
             # multiprocessing is not available or disabled, fallback
             # to sequential mode
@@ -264,29 +328,38 @@ class PoolManagerMixin(object):
             self._pool = None
 
     def _get_pool(self):
-        """Used by apply_async to make it possible to implement lazy init"""
+        """Used by `submit` to make it possible to implement lazy init"""
         return self._pool
 
-    def apply_async(self, func, callback=None):
+    def submit(self, func, callback=None):
         """Schedule a func to be run"""
         # Here, we need a wrapper to avoid crashes on KeyboardInterruptErrors.
         # We also call the callback on error, to make sure the pool does not
         # wait on crashed jobs.
         return self._get_pool().apply_async(
-            _TracebackCapturingWrapper(func), (),
-            callback=callback, error_callback=callback
+            _TracebackCapturingWrapper(func),
+            (),
+            callback=callback,
+            error_callback=callback,
         )
 
-    def retrieve_result_callback(self, out):
+    def retrieve_result_callback(self, result):
         """Mimic concurrent.futures results, raising an error if needed."""
-        return _retrieve_traceback_capturing_wrapped_call(out)
+        # In the multiprocessing Pool API, the callback are called with the
+        # result value as an argument so `result`(`out`) is the output of
+        # job.get(). It's either the result or the exception raised while
+        # collecting the result.
+        return _retrieve_traceback_capturing_wrapped_call(result)
 
     def abort_everything(self, ensure_ready=True):
         """Shutdown the pool and restart a new one with the same parameters"""
         self.terminate()
         if ensure_ready:
-            self.configure(n_jobs=self.parallel.n_jobs, parallel=self.parallel,
-                           **self.parallel._backend_args)
+            self.configure(
+                n_jobs=self.parallel.n_jobs,
+                parallel=self.parallel,
+                **self.parallel._backend_kwargs,
+            )
 
 
 class AutoBatchingMixin(object):
@@ -296,7 +369,7 @@ class AutoBatchingMixin(object):
     # overhead.
     # This settings was found by running benchmarks/bench_auto_batching.py
     # with various parameters on various platforms.
-    MIN_IDEAL_BATCH_DURATION = .2
+    MIN_IDEAL_BATCH_DURATION = 0.2
 
     # Should not be too high to avoid stragglers: long jobs running alone
     # on a single worker while other workers have no work to process any more.
@@ -315,14 +388,13 @@ class AutoBatchingMixin(object):
         """Determine the optimal batch size"""
         old_batch_size = self._effective_batch_size
         batch_duration = self._smoothed_batch_duration
-        if (batch_duration > 0 and
-                batch_duration < self.MIN_IDEAL_BATCH_DURATION):
+        if batch_duration > 0 and batch_duration < self.MIN_IDEAL_BATCH_DURATION:
             # The current batch size is too small: the duration of the
             # processing of a batch of task is not large enough to hide
             # the scheduling overhead.
-            ideal_batch_size = int(old_batch_size *
-                                   self.MIN_IDEAL_BATCH_DURATION /
-                                   batch_duration)
+            ideal_batch_size = int(
+                old_batch_size * self.MIN_IDEAL_BATCH_DURATION / batch_duration
+            )
             # Multiply by two to limit oscilations between min and max.
             ideal_batch_size *= 2
 
@@ -338,8 +410,7 @@ class AutoBatchingMixin(object):
                     f"Batch computation too fast ({batch_duration}s.) "
                     f"Setting batch_size={batch_size}."
                 )
-        elif (batch_duration > self.MAX_IDEAL_BATCH_DURATION and
-              old_batch_size >= 2):
+        elif batch_duration > self.MAX_IDEAL_BATCH_DURATION and old_batch_size >= 2:
             # The current batch size is too big. If we schedule overly long
             # running batches some CPUs might wait with nothing left to do
             # while a couple of CPUs a left processing a few long running
@@ -368,8 +439,7 @@ class AutoBatchingMixin(object):
             # CallBack as long as the batch_size is constant. Therefore
             # we need to reset the estimate whenever we re-tune the batch
             # size.
-            self._smoothed_batch_duration = \
-                self._DEFAULT_SMOOTHED_BATCH_DURATION
+            self._smoothed_batch_duration = self._DEFAULT_SMOOTHED_BATCH_DURATION
 
         return batch_size
 
@@ -417,13 +487,12 @@ class ThreadingBackend(PoolManagerMixin, ParallelBackendBase):
     uses_threads = True
     supports_sharedmem = True
 
-    def configure(self, n_jobs=1, parallel=None, **backend_args):
+    def configure(self, n_jobs=1, parallel=None, **backend_kwargs):
         """Build a process or thread pool and return the number of workers"""
         n_jobs = self.effective_n_jobs(n_jobs)
         if n_jobs == 1:
             # Avoid unnecessary overhead and use sequential backend instead.
-            raise FallbackToBackend(
-                SequentialBackend(nesting_level=self.nesting_level))
+            raise FallbackToBackend(SequentialBackend(nesting_level=self.nesting_level))
         self.parallel = parallel
         self._n_jobs = n_jobs
         return n_jobs
@@ -439,8 +508,7 @@ class ThreadingBackend(PoolManagerMixin, ParallelBackendBase):
         return self._pool
 
 
-class MultiprocessingBackend(PoolManagerMixin, AutoBatchingMixin,
-                             ParallelBackendBase):
+class MultiprocessingBackend(PoolManagerMixin, AutoBatchingMixin, ParallelBackendBase):
     """A ParallelBackend which will use a multiprocessing.Pool.
 
     Will introduce some communication and memory overhead when exchanging
@@ -475,8 +543,8 @@ class MultiprocessingBackend(PoolManagerMixin, AutoBatchingMixin,
                     )
                 else:
                     msg = (
-                        'Multiprocessing-backed parallel loops '
-                        'cannot be nested, setting n_jobs=1'
+                        "Multiprocessing-backed parallel loops "
+                        "cannot be nested, setting n_jobs=1"
                     )
                 warnings.warn(msg, stacklevel=3)
             return 1
@@ -485,33 +553,45 @@ class MultiprocessingBackend(PoolManagerMixin, AutoBatchingMixin,
             # Mixing loky and multiprocessing in nested loop is not supported
             if n_jobs != 1:
                 warnings.warn(
-                    'Multiprocessing-backed parallel loops cannot be nested,'
-                    ' below loky, setting n_jobs=1',
-                    stacklevel=3)
+                    "Multiprocessing-backed parallel loops cannot be nested,"
+                    " below loky, setting n_jobs=1",
+                    stacklevel=3,
+                )
             return 1
 
         elif not (self.in_main_thread() or self.nesting_level == 0):
             # Prevent posix fork inside in non-main posix threads
             if n_jobs != 1:
                 warnings.warn(
-                    'Multiprocessing-backed parallel loops cannot be nested'
-                    ' below threads, setting n_jobs=1',
-                    stacklevel=3)
+                    "Multiprocessing-backed parallel loops cannot be nested"
+                    " below threads, setting n_jobs=1",
+                    stacklevel=3,
+                )
             return 1
 
         return super(MultiprocessingBackend, self).effective_n_jobs(n_jobs)
 
-    def configure(self, n_jobs=1, parallel=None, prefer=None, require=None,
-                  **memmappingpool_args):
+    def configure(
+        self,
+        n_jobs=1,
+        parallel=None,
+        prefer=None,
+        require=None,
+        **memmapping_pool_kwargs,
+    ):
         """Build a process or thread pool and return the number of workers"""
         n_jobs = self.effective_n_jobs(n_jobs)
         if n_jobs == 1:
-            raise FallbackToBackend(
-                SequentialBackend(nesting_level=self.nesting_level))
+            raise FallbackToBackend(SequentialBackend(nesting_level=self.nesting_level))
+
+        memmapping_pool_kwargs = {
+            **self.backend_kwargs,
+            **memmapping_pool_kwargs,
+        }
 
         # Make sure to free as much memory as possible before forking
         gc.collect()
-        self._pool = MemmappingPool(n_jobs, **memmappingpool_args)
+        self._pool = MemmappingPool(n_jobs, **memmapping_pool_kwargs)
         self.parallel = parallel
         return n_jobs
 
@@ -527,25 +607,49 @@ class LokyBackend(AutoBatchingMixin, ParallelBackendBase):
     supports_retrieve_callback = True
     supports_inner_max_num_threads = True
 
-    def configure(self, n_jobs=1, parallel=None, prefer=None, require=None,
-                  idle_worker_timeout=300, **memmappingexecutor_args):
+    def configure(
+        self,
+        n_jobs=1,
+        parallel=None,
+        prefer=None,
+        require=None,
+        idle_worker_timeout=None,
+        **memmapping_executor_kwargs,
+    ):
         """Build a process executor and return the number of workers"""
         n_jobs = self.effective_n_jobs(n_jobs)
         if n_jobs == 1:
-            raise FallbackToBackend(
-                SequentialBackend(nesting_level=self.nesting_level))
+            raise FallbackToBackend(SequentialBackend(nesting_level=self.nesting_level))
+
+        memmapping_executor_kwargs = {
+            **self.backend_kwargs,
+            **memmapping_executor_kwargs,
+        }
+
+        # Prohibit the use of 'timeout' in the LokyBackend, as 'idle_worker_timeout'
+        # better describes the backend's behavior.
+        if "timeout" in memmapping_executor_kwargs:
+            raise ValueError(
+                "The 'timeout' parameter is not supported by the LokyBackend. "
+                "Please use the `idle_worker_timeout` parameter instead."
+            )
+        if idle_worker_timeout is None:
+            idle_worker_timeout = self.backend_kwargs.get("idle_worker_timeout", 300)
 
         self._workers = get_memmapping_executor(
-            n_jobs, timeout=idle_worker_timeout,
+            n_jobs,
+            timeout=idle_worker_timeout,
             env=self._prepare_worker_env(n_jobs=n_jobs),
-            context_id=parallel._id, **memmappingexecutor_args)
+            context_id=parallel._id,
+            **memmapping_executor_kwargs,
+        )
         self.parallel = parallel
         return n_jobs
 
     def effective_n_jobs(self, n_jobs):
         """Determine the number of jobs which are going to run in parallel"""
         if n_jobs == 0:
-            raise ValueError('n_jobs == 0 in Parallel has no meaning')
+            raise ValueError("n_jobs == 0 in Parallel has no meaning")
         elif mp is None or n_jobs is None:
             # multiprocessing is not available or disabled, fallback
             # to sequential mode
@@ -565,8 +669,8 @@ class LokyBackend(AutoBatchingMixin, ParallelBackendBase):
                     )
                 else:
                     msg = (
-                        'Loky-backed parallel loops cannot be called in a'
-                        ' multiprocessing, setting n_jobs=1'
+                        "Loky-backed parallel loops cannot be called in a"
+                        " multiprocessing, setting n_jobs=1"
                     )
                 warnings.warn(msg, stacklevel=3)
 
@@ -575,24 +679,26 @@ class LokyBackend(AutoBatchingMixin, ParallelBackendBase):
             # Prevent posix fork inside in non-main posix threads
             if n_jobs != 1:
                 warnings.warn(
-                    'Loky-backed parallel loops cannot be nested below '
-                    'threads, setting n_jobs=1',
-                    stacklevel=3)
+                    "Loky-backed parallel loops cannot be nested below "
+                    "threads, setting n_jobs=1",
+                    stacklevel=3,
+                )
             return 1
         elif n_jobs < 0:
             n_jobs = max(cpu_count() + 1 + n_jobs, 1)
         return n_jobs
 
-    def apply_async(self, func, callback=None):
+    def submit(self, func, callback=None):
         """Schedule a func to be run"""
         future = self._workers.submit(func)
         if callback is not None:
             future.add_done_callback(callback)
         return future
 
-    def retrieve_result_callback(self, out):
+    def retrieve_result_callback(self, future):
+        """Retrieve the result, here out is the future given by submit"""
         try:
-            return out.result()
+            return future.result()
         except ShutdownExecutorError:
             raise RuntimeError(
                 "The executor underlying Parallel has been shutdown. "
@@ -615,8 +721,7 @@ class LokyBackend(AutoBatchingMixin, ParallelBackendBase):
         self.reset_batch_stats()
 
     def abort_everything(self, ensure_ready=True):
-        """Shutdown the workers and restart a new one with the same parameters
-        """
+        """Shutdown the workers and restart a new one with the same parameters"""
         self._workers.terminate(kill_workers=True)
         self._workers = None
 
@@ -632,8 +737,7 @@ class FallbackToBackend(Exception):
 
 
 def inside_dask_worker():
-    """Check whether the current function is executed inside a Dask worker.
-    """
+    """Check whether the current function is executed inside a Dask worker."""
     # This function can not be in joblib._dask because there would be a
     # circular import:
     # _dask imports _parallel_backend that imports _dask ...
